@@ -2,17 +2,20 @@ import logging
 import uuid
 from abc import ABC, abstractmethod
 from collections.abc import Iterable
-from concurrent.futures import ProcessPoolExecutor, as_completed
+from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor, as_completed
 from pathlib import Path
 from typing import Any, cast
 
+import numpy as np
 from pyproj import CRS, Transformer
 from pyresample import create_area_def
 from pyresample.geometry import AreaDefinition, SwathDefinition
 from satpy.scene import Scene
 from shapely import Polygon
 
-from satctl.downloaders import Downloader
+from satctl.auth import AuthBuilder
+from satctl.auth.base import Authenticator
+from satctl.downloaders import DownloadBuilder, Downloader
 from satctl.model import ConversionParams, Granule, ProgressEventType, SearchParams
 from satctl.progress.events import emit_event
 from satctl.writers import Writer
@@ -26,7 +29,10 @@ class DataSource(ABC):
     def __init__(
         self,
         name: str,
-        downloader: Downloader,
+        auth_builder: AuthBuilder | None,
+        down_builder: DownloadBuilder | None,
+        default_authenticator: str | None = None,
+        default_downloader: str | None = None,
         default_resolution: int | None = None,
         default_composite: str | None = None,
     ):
@@ -34,16 +40,39 @@ class DataSource(ABC):
 
         Args:
             name (str): Source name identifier
-            downloader (Downloader): Downloader instance for retrieving files
+            auth_builder (AuthBuilder): Factory that creates an authenticator object on demand.
+            down_builder (DownloadBuilder): Factory that creates a downloader object on demand.
             default_resolution (int | None): Default resolution in meters. Defaults to None.
             default_composite (str | None): Default composite/dataset to load. Defaults to None.
+            default_authenticator (str | None): Default authenticator name to use when auth_builder is None. Defaults to None.
+            default_downloader (str | None): Default downloader name to use when down_builder is None. Defaults to None.
+
         """
         self.source_name = name
-        self.downloader = downloader
         self.default_composite = default_composite
         self.default_resolution = default_resolution
         self.reader = None
 
+        if auth_builder is None:
+            from satctl.auth import configure_authenticator
+
+            if default_authenticator is None:
+                raise ValueError(f"Authentication not configured for source: {name}")
+            auth_builder = configure_authenticator(default_authenticator)
+
+        if down_builder is None:
+            from satctl.downloaders import configure_downloader
+
+            if default_downloader is None:
+                raise ValueError(f"Downloader not configured for source: {name}")
+            down_builder = configure_downloader(default_downloader)
+
+        self.auth_builder = auth_builder
+        self.down_builder = down_builder
+        # lazy init to avoid attribute errors
+        self._authenticator: Authenticator | None = None
+        self._downloader: Downloader | None = None
+
     @property
     def collections(self) -> list[str]:
         """Get list of collection identifiers.
@@ -107,12 +136,14 @@ class DataSource(ABC):
         self,
         item: Granule,
         destination: Path,
+        downloader: Downloader,
     ) -> bool:
         """Download a single granule.
 
         Args:
             item (Granule): Granule to download
             destination (Path): Base destination directory
+            downloader (Downloader): Downloader instance to use for downloading
 
         Returns:
             bool: True if download succeeded, False otherwise
@@ -142,15 +173,15 @@ class DataSource(ABC):
         """
         ...
 
-    def get_downloader_init_kwargs(self) -> dict[str, Any]:
-        """Hook method for subclasses to provide downloader initialization arguments.
+    def authenticator(self, **overrides: dict) -> Authenticator:
+        if not self._authenticator:
+            self._authenticator = self.auth_builder(**overrides)
+        return cast(Authenticator, self._authenticator)
 
-        Override this method in subclasses to pass custom arguments to downloader.init().
-
-        Returns:
-            dict[str, Any]: Keyword arguments to pass to downloader.init()
-        """
-        return {}
+    def downloader(self, **overrides: dict) -> Downloader:
+        if not self._downloader:
+            self._downloader = self.down_builder(**overrides)
+        return cast(Downloader, self._downloader)
 
     def download(
         self,
@@ -184,11 +215,21 @@ class DataSource(ABC):
             total_items=len(items),
             description=self.collections[0],
         )
-        self.downloader.init(**self.get_downloader_init_kwargs())
+        # Initialize downloader
+        downloader = self.downloader()
+        downloader.init(self.authenticator())
         executor = None
         try:
-            with ProcessPoolExecutor(max_workers=num_workers) as executor:
-                future_to_item_map = {executor.submit(self.download_item, item, destination): item for item in items}
+            with ThreadPoolExecutor(max_workers=num_workers) as executor:
+                future_to_item_map = {
+                    executor.submit(
+                        self.download_item,
+                        item,
+                        destination,
+                        downloader,
+                    ): item
+                    for item in items
+                }
                 for future in as_completed(future_to_item_map):
                     item = future_to_item_map[future]
                     result = future.result()
@@ -499,7 +540,11 @@ class DataSource(ABC):
         Returns:
             dict[str, str]: Dictionary mapping dataset names to file names
         """
-        datasets_dict = writer.parse_datasets(params.datasets or self.default_composite)
+        datasets = params.datasets if params.datasets is not None else self.default_composite
+        if datasets is None:
+            raise ValueError("No datasets specified and no default composite configured")
+
+        datasets_dict = writer.parse_datasets(datasets)
         log.debug("Attempting to save the following datasets: %s", datasets_dict)
         return datasets_dict
 
@@ -576,6 +621,7 @@ class DataSource(ABC):
         destination: Path,
         granule_id: str,
         writer: Writer,
+        dtype: type | np.dtype[Any] | None = None,
     ) -> dict[str, list]:
         """Write all datasets from scene to output files.
 
@@ -603,6 +649,7 @@ class DataSource(ABC):
                 writer.write(
                     dataset=cast(DataArray, scene[dataset_name]),
                     output_path=output_path,
+                    dtype=dtype,
                 )
             )
         return paths
