@@ -9,9 +9,8 @@ from typing import Any
 import earthaccess
 from pydantic import BaseModel
 
-from satctl.auth.earthdata import EarthDataAuthenticator
-from satctl.downloaders import Downloader
-from satctl.downloaders.http import HTTPDownloader
+from satctl.auth import AuthBuilder
+from satctl.downloaders import DownloadBuilder, Downloader
 from satctl.model import ConversionParams, Granule, ProductInfo, SearchParams
 from satctl.sources import DataSource
 from satctl.writers import Writer
@@ -88,9 +87,12 @@ class EarthDataSource(DataSource):
         collection_name: str,
         *,
         reader: str,
-        downloader: Downloader,
         short_name: str,
         version: str | None = None,
+        auth_builder: AuthBuilder | None = None,
+        down_builder: DownloadBuilder | None = None,
+        default_authenticator: str | None = "earthdata",
+        default_downloader: str | None = "http",
         default_composite: str | None = None,
         default_resolution: int | None = None,
         search_limit: int = DEFAULT_SEARCH_LIMIT,
@@ -98,10 +100,13 @@ class EarthDataSource(DataSource):
         """Initialize EarthData source.
 
         Args:
-            collection_name (str): Collection name identifier
-            reader (str): Satpy reader name
-            downloader (Downloader): Downloader instance
-            short_name (str): NASA short name for the product
+            collection_name (str): Collection name identifier.
+            reader (str): Satpy reader name.
+            short_name (str): NASA short name for this source.
+            auth_builder (AuthBuilder): factory that creates an auth object on demand.
+            down_builder (DownloadBuilder): factory that create a downloader object on demand.
+            default_authenticator (str): Default authenticator name to use when auth_builder is None. Defaults to "earthdata".
+            default_downloader (str): Default downloader name to use when down_builder is None. Defaults to "http".
             version (str | None): Product version. Defaults to None.
             default_composite (str | None): Default composite name. Defaults to None.
             default_resolution (int | None): Default resolution in meters. Defaults to None.
@@ -109,9 +114,12 @@ class EarthDataSource(DataSource):
         """
         super().__init__(
             collection_name,
-            downloader=downloader,
+            auth_builder=auth_builder,
+            down_builder=down_builder,
             default_composite=default_composite,
             default_resolution=default_resolution,
+            default_authenticator=default_authenticator,
+            default_downloader=default_downloader,
         )
         self.reader = reader
         self.short_name = short_name
@@ -222,6 +230,19 @@ class EarthDataSource(DataSource):
         """
         ...
 
+    @abstractmethod
+    def _get_file_extension(self) -> str:
+        """Get the file extension for this data source.
+
+        Different sensors use different file formats:
+        - MODIS: HDF format (.hdf)
+        - VIIRS: NetCDF format (.nc)
+
+        Returns:
+            str: File extension without dot (e.g., "hdf", "nc")
+        """
+        ...
+
     def _get_short_name_from_granule(self, granule_id: str) -> str:
         """Extract the short_name from a granule ID.
 
@@ -254,6 +275,9 @@ class EarthDataSource(DataSource):
         Returns:
             list[Granule]: List of granules for this combination
         """
+        # Ensure authentication before searching (earthaccess requires global auth)
+        self.authenticator().ensure_authenticated()
+
         search_kwargs: dict[str, Any] = {
             "short_name": short_name,
             "temporal": (params.start.isoformat(), params.end.isoformat()),
@@ -304,52 +328,65 @@ class EarthDataSource(DataSource):
     def _get_granule_by_short_name(self, item_id: str, short_name: str) -> Granule:
         """Fetch a specific granule by ID and short_name.
 
+        Fetches both radiance (level 02) and georeference (level 03) data.
+
         Args:
             item_id (str): The granule ID
             short_name (str): NASA short name (e.g., "MOD02QKM", "VNP02MOD")
 
         Returns:
-            Granule: The requested granule
+            Granule: The requested granule with radiance and georeference assets
 
         Raises:
             ValueError: If granule not found
         """
         try:
-            results = earthaccess.search_data(
+            # Fetch radiance data (level 02)
+            extension = self._get_file_extension()
+            if not item_id.endswith(f".{extension}"):
+                item_id = f"{item_id}.{extension}"
+
+            radiance_results = earthaccess.search_data(
                 short_name=short_name,
                 granule_name=item_id,
             )
 
-            if not results:
+            if not radiance_results:
                 raise ValueError(f"No granule found with id: {item_id}")
-            item = results[0]
+            radiance_result = radiance_results[0]
+
+            # Get radiance ID - strip file extension
+            radiance_id_raw = radiance_result["umm"]["DataGranule"]["Identifiers"][0]["Identifier"]
+            radiance_id = ".".join(radiance_id_raw.split(".")[:-1])  # Agnostically remove file extension
+
+            # Find matching georeference file (level 03)
+            georeference_id_pattern = self._build_georeference_pattern(radiance_id)
+            georeference_short_name = self._get_georeference_short_name(short_name)
+
+            georeference_results = earthaccess.search_data(
+                short_name=georeference_short_name,
+                granule_name=georeference_id_pattern,
+            )
+
+            if not georeference_results:
+                raise ValueError(f"No georeference data found for granule: {radiance_id}")
+            georeference_result = georeference_results[0]
 
         except Exception as e:
             log.error("Failed to fetch granule %s: %s", item_id, e)
             raise
 
-        # Strip file extension from granule ID
-        item_id = item["umm"]["DataGranule"]["Identifiers"][0]["Identifier"].replace(".hdf", "").replace(".nc", "")
-
         return Granule(
-            granule_id=item_id,
+            granule_id=radiance_id,
             source=self.collections[0],
-            assets=parse_umm_assets(item, EarthDataAsset),
-            info=self._parse_item_name(item_id),
+            assets={
+                "radiance": parse_umm_assets(radiance_result, EarthDataAsset),
+                "georeference": parse_umm_assets(georeference_result, EarthDataAsset),
+            },
+            info=self._parse_item_name(radiance_id),
         )
 
-    def get_downloader_init_kwargs(self) -> dict[str, Any]:
-        """Provide EarthData session to downloader initialization.
-
-        Returns:
-            dict[str, Any]: Dictionary with session keyword argument if applicable
-        """
-        # Only provide session if we have HTTPDownloader with EarthDataAuthenticator
-        if isinstance(self.downloader, HTTPDownloader) and isinstance(self.downloader.auth, EarthDataAuthenticator):
-            return {"session": self.downloader.auth.auth_session}
-        return {}
-
-    def download_item(self, item: Granule, destination: Path) -> bool:
+    def download_item(self, item: Granule, destination: Path, downloader: Downloader) -> bool:
         """Download both radiance and georeference files for a granule.
 
         Args:
@@ -367,7 +404,7 @@ class EarthDataSource(DataSource):
         radiance_filename = Path(radiance_asset.href).name
         radiance_file = granule_dir / radiance_filename
 
-        radiance_success = self.downloader.download(
+        radiance_success = downloader.download(
             uri=radiance_asset.href,
             destination=radiance_file,
             item_id=item.granule_id,
@@ -382,7 +419,7 @@ class EarthDataSource(DataSource):
         georeference_filename = Path(georeference_asset.href).name
         georeference_file = granule_dir / georeference_filename
 
-        georeference_success = self.downloader.download(
+        georeference_success = downloader.download(
             uri=georeference_asset.href,
             destination=georeference_file,
             item_id=item.granule_id,
