@@ -1,6 +1,5 @@
 import logging
 import re
-import warnings
 from abc import abstractmethod
 from datetime import datetime, timezone
 from pathlib import Path
@@ -10,7 +9,8 @@ from pydantic import BaseModel
 from pystac_client import Client
 from satpy.scene import Scene
 
-from satctl.downloaders import Downloader
+from satctl.auth import AuthBuilder
+from satctl.downloaders import DownloadBuilder, Downloader
 from satctl.model import ConversionParams, Granule, ProductInfo, SearchParams
 from satctl.sources import DataSource
 from satctl.writers import Writer
@@ -60,39 +60,41 @@ class Sentinel2Source(DataSource):
         collection_name: str,
         *,
         reader: str,
-        downloader: Downloader,
         stac_url: str,
+        auth_builder: AuthBuilder | None,
+        down_builder: DownloadBuilder | None,
+        default_authenticator: str | None,
+        default_downloader: str | None,
         default_composite: str | None = None,
         default_resolution: int | None = None,
         search_limit: int = DEFAULT_SEARCH_LIMIT,
-        download_pool_conns: int = 10,
-        download_pool_size: int = 2,
     ):
         """Initialize Sentinel-2 source.
 
         Args:
             collection_name (str): Collection name identifier
             reader (str): Satpy reader name
-            downloader (Downloader): Downloader instance
             stac_url (str): STAC catalog URL
+            auth_builder (AuthBuilder | None): Factory that creates an authenticator object on demand. Defaults to None.
+            down_builder (DownloadBuilder | None): Factory that creates a downloader object on demand. Defaults to None.
+            default_authenticator (str | None): Default authenticator name to use when auth_builder is None.
+            default_downloader (str | None): Default downloader name to use when down_builder is None.
             default_composite (str | None): Default composite name. Defaults to None.
             default_resolution (int | None): Default resolution in meters. Defaults to None.
             search_limit (int): Maximum search results. Defaults to 100.
-            download_pool_conns (int): HTTP connection pool size. Defaults to 10.
-            download_pool_size (int): HTTP connection pool max size. Defaults to 2.
         """
         super().__init__(
             collection_name,
-            downloader=downloader,
+            auth_builder=auth_builder,
+            down_builder=down_builder,
+            default_downloader=default_downloader,
+            default_authenticator=default_authenticator,
             default_composite=default_composite,
             default_resolution=default_resolution,
         )
         self.reader = reader
         self.stac_url = stac_url
         self.search_limit = search_limit
-        self.download_pool_conns = download_pool_conns
-        self.download_pool_size = download_pool_size
-        warnings.filterwarnings(action="ignore", category=UserWarning)
 
     @abstractmethod
     def _parse_item_name(self, name: str) -> ProductInfo:
@@ -145,15 +147,35 @@ class Sentinel2Source(DataSource):
 
         Args:
             item_id (str): Granule identifier
-            **kwargs (Any): Additional keyword arguments
+            **kwargs (Any): Additional keyword arguments (unused)
 
         Returns:
             Granule: The requested granule
 
         Raises:
-            NotImplementedError: Method not yet implemented
+            ValueError: If granule not found
         """
-        raise NotImplementedError()
+        log.debug("Fetching Sentinel-2 granule by ID: %s", item_id)
+        catalogue = Client.open(self.stac_url)
+
+        try:
+            collection = catalogue.get_collection(self.collections[0])
+            stac_item = collection.get_item(item_id)
+            if stac_item is None:
+                raise ValueError(f"No granule found with id: {item_id}")
+        except Exception as e:
+            log.error("Failed to fetch granule %s: %s", item_id, e)
+            raise ValueError(f"No granule found with id: {item_id}") from e
+
+        return Granule(
+            granule_id=stac_item.id,
+            source=self.collections[0],
+            assets={
+                asset_name: S2Asset(href=asset.href, media_type=asset.media_type)
+                for asset_name, asset in stac_item.assets.items()
+            },
+            info=self._parse_item_name(stac_item.id),
+        )
 
     def get_files(self, item: Granule) -> list[Path | str]:
         """Get list of files from SAFE structure.
@@ -182,7 +204,7 @@ class Sentinel2Source(DataSource):
             all_files = [f for f in item.local_path.rglob("*") if f.is_file()]
             # Exclude _granule.json metadata file
             all_files = [f for f in all_files if f.name != "_granule.json"]
-            return all_files
+            return cast(list[Path | str], all_files)
         else:
             raise ValueError(
                 f"Invalid data: SAFE structure not found in '{item.local_path}' "
@@ -248,7 +270,7 @@ class Sentinel2Source(DataSource):
         scene.load(datasets, calibration=calibration)
         return scene
 
-    def download_item(self, item: Granule, destination: Path) -> bool:
+    def download_item(self, item: Granule, destination: Path, downloader: Downloader) -> bool:
         """Download required assets preserving SAFE directory structure.
 
         Args:
@@ -287,7 +309,8 @@ class Sentinel2Source(DataSource):
                 target_file = local_path / (asset_name + Path(asset.href).suffix)
 
             target_file.parent.mkdir(parents=True, exist_ok=True)
-            result = self.downloader.download(
+
+            result = downloader.download(
                 uri=asset.href,
                 destination=target_file,
                 item_id=item.granule_id,
@@ -314,7 +337,8 @@ class Sentinel2Source(DataSource):
                 target_file = local_path / Path(metadata.href).name
 
             target_file.parent.mkdir(parents=True, exist_ok=True)
-            result = self.downloader.download(
+
+            result = downloader.download(
                 uri=metadata.href,
                 destination=target_file,
                 item_id=item.granule_id,
@@ -352,21 +376,16 @@ class Sentinel2Source(DataSource):
         """
         # Validate inputs using base class helper
         self._validate_save_inputs(item, params)
-
         # Parse datasets using base class helper
         datasets_dict = self._prepare_datasets(writer, params)
-
         # Filter existing files using base class helper
         datasets_dict = self._filter_existing_files(datasets_dict, destination, item.granule_id, writer, force)
-
         # Load and resample scene
         log.debug("Loading and resampling scene")
         scene = self.load_scene(item, datasets=list(datasets_dict.values()))
-
         # Define area using base class helper
         area_def = self._create_area_from_params(params, scene)
         scene = self.resample(scene, area_def=area_def)
-
         # Write datasets using base class helper
         return self._write_scene_datasets(scene, datasets_dict, destination, item.granule_id, writer)
 
@@ -402,35 +421,38 @@ class Sentinel2L2ASource(Sentinel2Source):
     def __init__(
         self,
         *,
-        downloader: Downloader,
         stac_url: str,
+        auth_builder: AuthBuilder | None = None,
+        down_builder: DownloadBuilder | None = None,
+        default_authenticator: str | None = "s3",
+        default_downloader: str | None = "s3",
         default_composite: str = "true_color",
         default_resolution: int = 10,
         search_limit: int = 100,
-        download_pool_conns: int = 10,
-        download_pool_size: int = 2,
     ):
         """Initialize Sentinel-2 L2A source.
 
         Args:
-            downloader (Downloader): Downloader instance
             stac_url (str): STAC catalog URL
+            auth_builder (AuthBuilder | None): Factory that creates an authenticator object on demand. Defaults to None.
+            down_builder (DownloadBuilder | None): Factory that creates a downloader object on demand. Defaults to None.
+            default_authenticator (str | None): Default authenticator name to use when auth_builder is None. Defaults to "s3".
+            default_downloader (str | None): Default downloader name to use when down_builder is None. Defaults to "s3".
             default_composite (str): Default composite name. Defaults to 'true_color'.
             default_resolution (int): Default resolution in meters. Defaults to 10.
             search_limit (int): Maximum search results. Defaults to 100.
-            download_pool_conns (int): HTTP connection pool size. Defaults to 10.
-            download_pool_size (int): HTTP connection pool max size. Defaults to 2.
         """
         super().__init__(
             "sentinel-2-l2a",
             reader="msi_safe_l2a",
+            auth_builder=auth_builder,
+            down_builder=down_builder,
+            default_downloader=default_downloader,
+            default_authenticator=default_authenticator,
             default_composite=default_composite,
             default_resolution=default_resolution,
-            downloader=downloader,
             stac_url=stac_url,
             search_limit=search_limit,
-            download_pool_conns=download_pool_conns,
-            download_pool_size=download_pool_size,
         )
 
     def _parse_item_name(self, name: str) -> ProductInfo:
@@ -492,35 +514,38 @@ class Sentinel2L1CSource(Sentinel2Source):
     def __init__(
         self,
         *,
-        downloader: Downloader,
         stac_url: str,
+        auth_builder: AuthBuilder | None = None,
+        down_builder: DownloadBuilder | None = None,
+        default_authenticator: str | None = "s3",
+        default_downloader: str | None = "s3",
         default_composite: str = "true_color",
         default_resolution: int = 10,
         search_limit: int = 100,
-        download_pool_conns: int = 10,
-        download_pool_size: int = 2,
     ):
         """Initialize Sentinel-2 L1C source.
 
         Args:
-            downloader (Downloader): Downloader instance
             stac_url (str): STAC catalog URL
+            auth_builder (AuthBuilder | None): Factory that creates an authenticator object on demand. Defaults to None.
+            down_builder (DownloadBuilder | None): Factory that creates a downloader object on demand. Defaults to None.
+            default_authenticator (str | None): Default authenticator name to use when auth_builder is None. Defaults to "s3".
+            default_downloader (str | None): Default downloader name to use when down_builder is None. Defaults to "s3".
             default_composite (str): Default composite name. Defaults to 'true_color'.
             default_resolution (int): Default resolution in meters. Defaults to 10.
             search_limit (int): Maximum search results. Defaults to 100.
-            download_pool_conns (int): HTTP connection pool size. Defaults to 10.
-            download_pool_size (int): HTTP connection pool max size. Defaults to 2.
         """
         super().__init__(
             "sentinel-2-l1c",
             reader="msi_safe",
+            auth_builder=auth_builder,
+            down_builder=down_builder,
+            default_authenticator=default_authenticator,
+            default_downloader=default_downloader,
             default_composite=default_composite,
             default_resolution=default_resolution,
-            downloader=downloader,
             stac_url=stac_url,
             search_limit=search_limit,
-            download_pool_conns=download_pool_conns,
-            download_pool_size=download_pool_size,
         )
 
     def _parse_item_name(self, name: str) -> ProductInfo:
